
/******************************************************************/
/* Cplx.h                                                         */
/* Stefan Bauberger (stefan@bauberger.de)                         */
/* Ayres Freitas (afreitas@pitt.edu)                              */
/* last revision 30.07.15                                         */
/******************************************************************/
/* interface to template for the definition of complex numbers    */
/* and to the the quad-package for calculation with double-double */
/* precision for use with the package s2lse                       */
/* (scalar two-loop self-energy integrals)                        */
/******************************************************************/

#ifndef __Cplx__
#define __Cplx__

#include <stdlib.h>

#include <fstream>

#define TRUE  1
#define FALSE 0

#ifdef QUADPRECISION

#include "doubledouble/doubledouble.h"
// #include <std/complext.cc>
#include <complex>

template <class FLOAT> complex<FLOAT>
sqr(const complex<FLOAT>& x) { return x*x; }

template <class FLOAT> complex<FLOAT>
powint(const complex<FLOAT>& x, int i)
{
  int j=i;
  if (j<0) j=-j;
  complex<FLOAT> res=complex<FLOAT>(1);
  switch(j) {
    case 0: break;
    case 1: res=x; break;
    case 2: res=x*x; break;
    default: 
      int n=j/2;
      if (n+n != j) res=x*sqr(powint(x,n));
      else res=sqr(powint(x,n));
      break ; };
  if (i>=0) return(res);
  else return(1/res);
} 

typedef complex<doubledouble> Cplx;

inline Cplx operator + (int x, const Cplx& y)
{ return(Cplx(x)+y); }
inline Cplx operator + (double x, const Cplx& y)
{ return(Cplx(x)+y); }
inline Cplx operator + (const Cplx& x,int y)
{ return(x+Cplx(y)); }
inline Cplx operator + (const Cplx& x,double y)
{ return(x+Cplx(y)); }

inline Cplx operator - (int x, const Cplx& y)
{ return(Cplx(x)-y); }
inline Cplx operator - (double x, const Cplx& y)
{ return(Cplx(x)-y); }
inline Cplx operator - (const Cplx& x,int y)
{ return(x-Cplx(y)); }
inline Cplx operator - (const Cplx& x,double y)
{ return(x-Cplx(y)); }

inline Cplx operator * (int x, const Cplx& y)
{ return(Cplx(x)*y); }
inline Cplx operator * (double x, const Cplx& y)
{ return(Cplx(x)*y); }
inline Cplx operator * (const Cplx& x,int y)
{ return(x*Cplx(y)); }
inline Cplx operator * (const Cplx& x,double y)
{ return(x*Cplx(y)); }

inline Cplx operator / (int x, const Cplx& y)
{ return(Cplx(x)/y); }
inline Cplx operator / (double x, const Cplx& y)
{ return(Cplx(x)/y); }
inline Cplx operator / (const Cplx& x,int y)
{ return(x/Cplx(y)); }
inline Cplx operator / (const Cplx& x,double y)
{ return(x/Cplx(y)); }

inline bool operator == (const Cplx& x, double y)
{ return(x==Cplx(y)); }
inline bool operator == (const Cplx& x, int y)
{ return(x==Cplx(y)); }
inline bool operator == (double x, const Cplx& y)
{ return(Cplx(x)==y); }
inline bool operator == (int x, const Cplx& y)
{ return(Cplx(x)==y); }


inline bool operator != (const Cplx& x, double y)
{ return(x!=Cplx(y)); }
inline bool operator != (const Cplx& x, int y)
{ return(x!=Cplx(y)); }
inline bool operator != (double x, const Cplx& y)
{ return(Cplx(x)!=y); }
inline bool operator != (int x, const Cplx& y)
{ return(Cplx(x)!=y); }
    
/*
inline doubledouble operator +(const doubledouble& x,double y) { return(x+doubledouble(y)); }
inline doubledouble operator +(const doubledouble& x,int y) { return(x+doubledouble(y)); }
inline doubledouble operator +(int x,const doubledouble& y) { return(doubledouble(x)+y); }
inline doubledouble operator -(const doubledouble& x,double y) { return(x+doubledouble(-y)); }
inline doubledouble operator -(const doubledouble& x,int y) { return(x+doubledouble(-y)); }
inline doubledouble operator -(int x,const doubledouble& y) { return(doubledouble(x)-y); }
inline doubledouble operator *(const doubledouble& x,double y) { return(x*doubledouble(y)); }
inline doubledouble operator *(const doubledouble& x,int y) { return(x*doubledouble(y)); }
inline doubledouble operator /(int x,const doubledouble& y) { return(doubledouble(x)/y); }
*/
inline bool operator ==(const doubledouble& x,double y) { return(x.h() == y); }
inline bool operator ==(double x,const doubledouble& y) { return(x == y.h()); }
inline bool operator ==(const doubledouble& x,int y) { return(x.h() == y); }
inline bool operator ==(int x,const doubledouble& y) { return(x == y.h()); }
inline bool operator >(const doubledouble& x,double y) { return(x.h() > y); }
inline bool operator >(double x,const doubledouble& y) { return(x > y.h()); }
inline bool operator >(const doubledouble& x,int y) { return(x.h() > y); }
inline bool operator >(int x,const doubledouble& y) { return(x > y.h()); }
inline bool operator <(const doubledouble& x,double y) { return(x.h() < y); }
inline bool operator <(double x,const doubledouble& y) { return(x < y.h()); }
inline bool operator <(const doubledouble& x,int y) { return(x.h() < y); }
inline bool operator <(int x,const doubledouble& y) { return(x < y.h()); }
inline bool operator >=(const doubledouble& x,double y) { return(x.h() >= y); }
inline bool operator >=(double x,const doubledouble& y) { return(x >= y.h()); }
inline bool operator >=(const doubledouble& x,int y) { return(x.h() >= y); }
inline bool operator >=(int x,const doubledouble& y) { return(x >= y.h()); }
inline bool operator <=(const doubledouble& x,double y) { return(x.h() <= y); }
inline bool operator <=(double x,const doubledouble& y) { return(x <= y.h()); }
inline bool operator <=(const doubledouble& x,int y) { return(x.h() <= y); }
inline bool operator <=(int x,const doubledouble& y) { return(x <= y.h()); }
inline bool operator !=(const doubledouble& x,double y) { return(x.h() != y || x.l() != 0.); }
inline bool operator !=(double x,const doubledouble& y) { return(x != y.h() || y.h() != 0.); }
inline bool operator !=(const doubledouble& x,int y) { return(x.h() != y || x.l() != 0.); }
inline bool operator !=(int x,const doubledouble& y) { return(x != y.h() || y.h() != 0.); }

#define Double doubledouble
#define SDouble doubledouble

static Cplx SCplx(char *s1,char *s2) 
{
 doubledouble q1(s1),q2(s2); return(Cplx(q1,q2));
}

static Cplx RCplx(Double r) { return(Cplx(r)); }
static Cplx ICplx(int i) { return(Cplx(i)); }

#define Pi doubledouble::Pi
#define TwoPi doubledouble::TwoPi

static const Double PiSonSix=SDouble("1.64493406684822643647241516664603"); 
                                    // pi^2/6
static const Double PiSonThree=SDouble("3.28986813369645287294483033329205"); 
                                    // pi^2/3
static const Cplx PiI(doubledouble(0.),Pi);
static const Cplx TwoPiI(doubledouble(0.),TwoPi);
static const Cplx I(doubledouble(0.),doubledouble(1.));
static const doubledouble EULERGAMMA("0.577215664901532860606512090082402");
static const doubledouble EulerGamma=EULERGAMMA;

#else  // QUADPRECISION

#include<complex.h>

#define Double double
#if (__GNUC_MINOR__==6)
#define Cplx complex
#define double_complex complex
#else
#define Cplx double_complex
#endif

inline Cplx sqr(const Cplx& x) { return x*x; }

#define powint pow

inline Double SDouble(char *s1) { return(atof(s1)); }
inline Cplx SCplx(char *s1,char *s2) { return(Cplx(atof(s1),atof(s2))); }
inline Cplx RCplx(Double r) { return(Cplx(r)); }
inline Cplx ICplx(int i) { return(Cplx(i)); }

#define Pi 3.141592653589793
#define TwoPi 6.283185307179587
#define PiSonSix 1.644934066848226  // pi^2/6
#define PiSonThree 3.289868133696452 // pi^2/3
#define EULERGAMMA 0.5772156649015329
#define EulerGamma 0.5772156649015329
static const Cplx PiI(0.,Pi);
static const Cplx TwoPiI(0.,TwoPi);
static const Cplx I(0.,1.);

#endif // #else QUADPRECISION

inline Double abs(Double x) { return(fabs(x)); }

#endif // __Cplx__
