/* ------------------------------------------------------------- */
/* U45.cc                                                        */
/* Ayres Freitas (afreitas@pitt.edu)			         */
/* last revision 06.12.16                                        */
/* ------------------------------------------------------------- */
/* 3-loop master integrals with four (U4) and five (U5)          */
/* independent propagators (subtracted finite parts)  	         */
/* ------------------------------------------------------------- */
/* Reference:                                                    */
/*   [1] = Freitas, arXiv:1609.09159, JHEP 1611 (2016) 145       */
/* See also:                                                     */
/*   [2] = Bauberger, Berends, Boehm, Buza,                      */
/*         hep-ph/9409388, Nucl.Phys.B 343 (1995) 383            */
/* ------------------------------------------------------------- */

#include "U456.h"
#include "li.h"
#include "qagutil.h"
#include "B0.h"

/******************************************************************/
/* some constants which define from which value on the asymptotic */
/* limit of some functions will be taken:                         */
/******************************************************************/

#ifdef QUADPRECISION
static Double T11234ASYMPLIMIT1=SDouble("1e-14");
#else
#define T11234ASYMPLIMIT1 1e-4
#endif

// zeta(n):
#define MAX_ZETA 5
static Double zeta[MAX_ZETA+1] =
{ SDouble("0"),SDouble(" 0"),
  SDouble(" 1.64493406684822643647241516665"),
  SDouble("1.202056903159594285399738161511"),
  SDouble(" 1.08232323371113819151600369654"),
  SDouble("1.036927755143369926331365486457") };

#define Zeta zeta

#define Errstream cerr

/******************************************************************/
/* Throughout this file the numbers are defined as Double or Cplx   */
/* to enable a generalization to arbitrary precision              */
/******************************************************************/

/******************************************************************/
/* convention: vars is frequently used for (var)^2                */
/* for example: m1s denotes m_1^2.                                */
/******************************************************************/

/******************************************************************/
/* routines which cry for improvement are marked with             */
/* "TO IMPROVE".                                                  */
/******************************************************************/

// discontinuity of B0
inline Double DeltaB0(Double s, Double m1se, Double m2se)
{ if(s > sqr(sqrt(m1se)+sqrt(m2se)))
    return(sla(s,m1se,m2se)/s);
  else
    return(0);
}

// discontinuity of B0(s,m1s,m2s)-B0(s,0,0), above threshold
inline Double DeltaB0sub(Double s, Double m1se, Double m2se)
{ return((sqr(m1se-m2se)-2*s*(m1se+m2se))/(s*(sla(s,m1se,m2se)+s)));
}

// discontinuity of derivative of B0 with respect to m1s
inline Double DM1DeltaB0(Double s, Double m1se, Double m2se)
{ if(s > sqr(sqrt(m1se)+sqrt(m2se)))
    return((m1se-m2se-s)/(s*sla(s,m1se,m2se)));
  else
    return(0);
}

// discontinuity of derivative of B0(s,m1s,m2s)-B0(s,m1s,0) with respect to m1s,
//  above threshold
inline Double DM1DeltaB0sub(Double s, Double m1se, Double m2se)
{ return(4*m2se/((m1se-m2se-s)*sla(s,m1se,m2se)-la(s,m1se,m2se)));
}

// real part of B0(s,m1s,m2s)-B0(s,0,0)
Double B0sub(Double s, Double m1se, Double m2se)
{
  if(m1se==Double(0) && m2se==Double(0)) return(0);
  if(m1se==Double(0)) interchange(&m1se,&m2se);
  if(m2se==Double(0))
    return(m1se/s*log(s/m1se) - (1-m1se/s)*log(abs(1-m1se/s)));
  Cplx vsla = csqrt1(la(s,m1se,m2se))/s;
  return(real((m1se-m2se)/(2*s)*log(m2se/m1se)
              - (sqr(m1se/s-m2se/s)-2*(m1se+m2se)/s)/(vsla+1)
		*(log(sqr(s)/(m1se*m2se))/2 - PiI)
	      - vsla*log((1-m1se/s-m2se/s+vsla)/2)));
}

Double DM1B0h(Double s, Double m1se, Double m2se)
{ 
  if(m2se==Double(0))
    return((B0sub(s,m1se,m2se)-log(s)+log(m1se))/(m1se-s));
  else
    return(((m1se-m2se-s)*(B0sub(s,m1se,m2se)-log(s))
            + (m1se+m2se-s)*log(m1se) - 2*m2se*log(m2se)) / la(s,m1se,m2se));
}

static Double m1s,m2s,m3s,m4s,m5s,sa,sb;
static Cplx U4k(Double x);
static Cplx U4hk(Double x);
static Cplx U5k(Double x);


static Cplx U4k(Double s)
{
  return(s*(1-log(s))*(DM1DeltaB0(s,m1s,m2s)*B0sub(s,m3s,m4s) -
                       DM1DeltaB0(s,m1s,0)*(B0sub(s,m3s,0)+B0sub(s,m4s,0)) +
		       (DeltaB0(s,m3s,m4s)-1)*DM1B0h(s,m1s,m2s) -
		       (DeltaB0(s,m3s,0)+DeltaB0(s,m4s,0)-2)
		         *DM1B0h(s,m1s,0)));
}

static Cplx U4ki(Double s)
{ return(U4k(2*sa-s)); }

static Cplx U4hk(Double s)
{
  if(s/(m1s+m2s+m3s+m4s) > SDouble("1e10"))
    // use large-s expansion of 
    // DM1DeltaB0(s,m1s,0)*(B0sub(s,m3s,m4s)-B0sub(s,m3s,0)-B0sub(s,m4s,0))
    return(s*(1-log(s))*(-(m3s*m4s/(s*s*s)*(log(sqr(s)/(m3s*m4s)) 
   			   + (m3s+m4s)/(2*s)*(2*log(sqr(s)/(m3s*m4s))-3))) +
			 DM1DeltaB0sub(s,m1s,m2s)*B0sub(s,m3s,m4s) +
		         DeltaB0sub(s,m3s,m4s)*DM1B0h(s,m1s,m2s) -
			 (DeltaB0sub(s,m3s,0)+DeltaB0sub(s,m4s,0))
		           *DM1B0h(s,m1s,0)));
  else	   
  return(s*(1-log(s))*(DM1DeltaB0(s,m1s,m2s)*B0sub(s,m3s,m4s) -
                       DM1DeltaB0(s,m1s,0)*(B0sub(s,m3s,0)+B0sub(s,m4s,0)) +
		       DeltaB0sub(s,m3s,m4s)*DM1B0h(s,m1s,m2s) -
		       (DeltaB0sub(s,m3s,0)+DeltaB0sub(s,m4s,0))
		         *DM1B0h(s,m1s,0)));
}


Double U4int(Double m1se, Double m2se, Double m3se, Double m4se,
             Double prec, Double *abserr)
{
  Double res, err1, err2, err3;
  m1s=m1se; m2s=m2se; m3s=m3se; m4s=m4se;
  if(m3s < m2s) interchange(&m3s,&m2s);
  if(m4s < m2s) interchange(&m4s,&m2s);
  sa = sqr(sqrt(m1s)+sqrt(m2s));
  sb = sqr(sqrt(m3s)+sqrt(m4s));
  if(sa > sb) interchange(&sa,&sb);
  res = -real(Cqagasqrtsing(U4ki,sa,2*sa,prec,&err1) +
              Cqagasqrtsing(U4k,sa,sb,prec,&err2) +
	      Cqaginf(U4hk,sb,prec,&err3));
  *abserr = sqrt(sqr(err1)+sqr(err2)+sqr(err3));
  return(res);
}


inline Double A0fin(Double ms)
{ return(ms*(1-log(ms)));
}

static Cplx U40k(Double s)
{
  Double res = (DM1DeltaB0(s,0,m2s)*A0fin(s)*B0sub(s,m3s,m4s)*(s-m2s) -
         DM1DeltaB0(s,0,m2s)*A0fin(m2s)*m2s/s*B0sub(m2s,m3s,m4s)*(s-m2s) -
	 DM1DeltaB0(s,0,0)*A0fin(s)*(B0sub(s,m3s,0)+B0sub(s,m4s,0))*(s-m2s) +
	 (DeltaB0(s,m3s,m4s)-1)*A0fin(s)*(1+m2s/s)*log(abs(1-s/m2s)) -
	 (DeltaB0(s,m3s,0)+DeltaB0(s,m4s,0)-2)*A0fin(s)*log(s/m2s)/s*(s-m2s));
//  cout << s << " " << DM1DeltaB0(s,0,m2s)*A0fin(m2s)*m2s/s*B0sub(m2s,m3s,m4s) << endl;
  return(res);
}

Double U40int(Double m2se, Double m3se, Double m4se,
              Double prec, Double *abserr)
{
  Double res, err1, err2, err3;
  m2s=m2se; m3s=m3se; m4s=m4se;
//  if(m3s < m2s) interchange(&m3s,&m2s);
//  if(m4s < m2s) interchange(&m4s,&m2s);
//  sa = m2s;
//  cout << U40k(0.6)/(0.6-sa) << " " << U40k(1.5)/(1.5-sa) << " " << 
//   U40k(3.)/(3.-sa) << endl;
  res = -real(Cqagpinfasqrtsing(U40k,0,m2s,0,prec,abserr));
  res += A0fin(m2s) * (-B0sub(m2s,m3s,m4s) + 
  			sqr(Pi)*(DeltaB0(m2s,m3s,m4s) - DeltaB0(m2s,0,0)));
  return(res);
}


// return x*log(x)
inline Double llog(Double x)
{
  if(x==Double(0))
    return(Double(0));
  else
    return(x*log(x));
}

static Cplx U5k(Double s)
{
  return((m5s-s-llog(m5s)+llog(s))/(m5s-s) *
         (DM1DeltaB0(s,m1s,m2s)*DM1B0h(s,m4s,m3s) +
	  DM1DeltaB0(s,m4s,m3s)*DM1B0h(s,m1s,m2s)));
}

static Cplx U5ki(Double s)
{ return(U5k(2*sb-s)); }

// modified integration kernel for the case sa==sb;
// it corresponds to U5 - (m5s-sa-m5s*log(m5s)+sa*log(sa))*
//                         real(DM1B0(m5s,m1s,m2s)*DM1B0(m5s,m4s,m3s));
// and it ensures that the integral \int_sa^sb ds U5ek(s) vanishes
// in the limit sb->sa;
// the term with the product of two DM1B0 functions needs to be added back
// explicitly to the integral
static Cplx U5ek(Double s)
{
  return((sa-s+llog(s)-llog(sa)) *
         (DM1DeltaB0(s,m1s,m2s)*DM1B0h(s,m4s,m3s) +
	  DM1DeltaB0(s,m4s,m3s)*DM1B0h(s,m1s,m2s)));
}


Double U5int(Double m1se, Double m2se, Double m3se, Double m4se, Double m5se,
             Double prec, Double *abserr)
{
  Double res, err1, err2, err3;
  m1s=m1se; m2s=m2se; m3s=m3se; m4s=m4se; m5s=m5se;
  sa = sqr(sqrt(m1s)+sqrt(m2s));
  sb = sqr(sqrt(m3s)+sqrt(m4s));
  if(sa > sb) interchange(&sa,&sb);
  if(sa != sb)
  {
    res = -real(Cqagasqrtsing(U5k,sa,(sa+sb)/2,prec,&err1) +
        	Cqagasqrtsing(U5ki,sb,(3*sb-sa)/2,prec,&err2) +
		Cqaginfasqrtsing(U5k,sb,prec,&err3));
    *abserr = sqrt(sqr(err1)+sqr(err2)+sqr(err3));
  }
  else
  {
    res = (m5s-sa-llog(m5s)+llog(sa))*
    		real(DM1B0(m5s,m1s,m2s)*DM1B0(m5s,m4s,m3s));
    res += real(Cqagpinfasqrtsing(U5ek,sb,m5s,0,prec,abserr));
  }
  return(res);
}


static Cplx U50k(Double s)
{
  return((m5s-s-llog(m5s)+llog(s))/(m5s-s) *
         ((DeltaB0(s,m4s,0)-1)*DM1B0h(s,m4s,m3s)/m4s +
	  DM1DeltaB0(s,m4s,m3s)*B0sub(s,m4s,0)/m4s));
}

static Cplx U50ki(Double s)
{ return(U50k(2*sb-s)); }

Double U50int(Double m3se, Double m4se, Double m5se,
              Double prec, Double *abserr)
{
  Double res, err1, err2, err3;
  m3s=m3se; m4s=m4se; m5s=m5se;
  sa = 0;
  sb = sqr(sqrt(m3s)+sqrt(m4s));
  res = -real(Cqagasqrtsing(U50k,sa,(sa+sb)/2,prec,&err1) +
              Cqagasqrtsing(U50ki,sb,(3*sb-sa)/2,prec,&err2) +
	      Cqaginfasqrtsing(U50k,sb,prec,&err3));
  *abserr = sqrt(sqr(err1)+sqr(err2)+sqr(err3));
  return(res);
}


